const AWS = require('aws-sdk')
const moment = require('moment')
const _ = require('lodash')
const getLogStreams = require('../utils/aws/getLogStreams')
const formatLogs = require('../utils/aws/formatLogs')
const getConfig = require('../utils/getConfig')

let startTime
let nextToken

const showLogs = async (logStreamNames) => {
  const config = await getConfig()
  const CloudWatchLogs = new AWS.CloudWatchLogs({region: config.region})
  const logGroupName = `/aws/lambda/${config.name}`
  if (!logStreamNames || !logStreamNames.length) {
    return setTimeout(() => getLogStreams(config)
        .then(nextLogStreamNames => showLogs(nextLogStreamNames)),
      500)
  }

  const params = {
    logGroupName: logGroupName,
    interleaved: true,
    logStreamNames
  }

  if (nextToken) params.nextToken = nextToken
  if (startTime) {
    const since = (['m', 'h', 'd']
      .indexOf(startTime[startTime.length - 1]) !== -1)
    if (since) {
      params.startTime = moment().subtract(startTime.replace(/\D/g, ''), startTime.replace(/\d/g, '')).valueOf()
    } else {
      params.startTime = moment.utc(startTime).valueOf()
    }
  }

  const results = await CloudWatchLogs.filterLogEvents(params).promise()
  if (results.events) {
    results.events.forEach(e => {
      process.stdout.write(formatLogs(e.message))
    })
  }

  if (results.nextToken) {
    nextToken = results.nextToken
  } else {
    nextToken = null
  }

  if (results.events && results.events.length) {
    startTime = _.last(results.events).timestamp + 1
  }

  return setTimeout(() => getLogStreams(config)
      .then(nextLogStreamNames => showLogs(nextLogStreamNames)),
    500)
}
module.exports = showLogs
